%OUTPUT
%Ab: final population
%f: fitness vector of the final population
%w: best solution

%INPUT
%alpha: percent of new individuals to be inserted
%beta: clone numbers parameter
%rho: mutation parameter
%Nn: population size
%dm: individual size
%Ngen: number of iterations
%fitfunc: fitness function handler
%mode: operation mode - 'falante' ou 'quieto'

%CLONALG para otimizao contnua multimodal
function [solu,Ab,f] = clonalg_wh(fitfunc,alpha,beta,rho,Nn,dm,Ngen,mode)

global MAX_g MIN_g MAX_FIR MIN_FIR MAX_IIR MIN_IIR;

MAX_g = 1;
MIN_g = 1e-12;
MAX_FIR = 1;
MIN_FIR = -1;
MAX_IIR = 4;
MIN_IIR = -4;

%initial population
d = round(alpha*Nn);
Ab = generate(Nn,dm);

if(strcmp('falante',mode))
    figure(1);
    fithist = zeros(2,Ngen);
end

for itg=1:Ngen

    f = feval(fitfunc,Ab);%fitness evaluation
    fnorm = normfit(f);
    if(strcmp('falante',mode))
        fithist(:,itg) = [max(f);mean(f)];
        fprintf('Iteration %d, best fitness: %f, avg. fitness: %f\n',itg,max(f),mean(f));
    end
    %%%%%%%%%%%%cloning_mutation_selection%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Nc = max([1 round(beta*Nn)]); %clone number
    C = allocate_pop(Nn); %clone population to be selected
    f_c = zeros(Nn,1); %clones fitness

    for it=1:Nn %for each individual in Ab

        Ctmp = repmat(Ab(it),1,Nc+1); %Nc copies of Ab + parent cell
        ftmp = f(it)*ones(Nc+1,1);
        ftmp_norm = fnorm(it)*ones(Nc+1,1);%normalized fitness in [0,1]

        Ctmp(1:Nc) = hypermut(Ctmp(1:Nc),ftmp_norm(1:Nc),rho,dm); %Nc clones maturation
        ftmp(1:Nc) = feval(fitfunc,Ctmp(1:Nc)); %clones' fitness is updated

        [best,bestidx] = max(ftmp);

        C(it) = Ctmp(bestidx); %the best fitness clone (or the parent) is selected
        f_c(it) = best;
    end

    Ab = C; %new population
    f = f_c;

    %new individuals replace the worst ones
    Abd = generate(d,dm);
    Ab = replace(Ab,Abd,f);%
    
    if(mod(itg,10)==0 && (strcmp('falante',mode)))
        figure(1);plot(1:itg,fithist(1,1:itg),'.',1:itg,fithist(2,1:itg),'.');
        legend('best','mean');hold on;
    end
        
end

%end of search process, best individual is selected
f = feval(fitfunc,Ab);
[~,indmax] = max(f);
solu = Ab(indmax);

end

%P: new population
%N: number of individuals
%dm: vector of dimensions - dm(1) corresponds to the FIR part, 
% dm(2) corresponds to the IIR part and dm(3) is the polynomial part
function P = generate(N,dm)

global MAX_g MIN_g MAX_FIR MIN_FIR MAX_IIR MIN_IIR;

P = allocate_pop(N);

%individuals generation
for it=1:N

    P(it).FIR = MIN_FIR + (MAX_FIR - MIN_FIR).*rand(1,dm(1));
    P(it).IIR = MIN_IIR + (MAX_IIR - MIN_IIR).*rand(1,dm(2));
    P(it).g = MIN_g + (MAX_g - MIN_g).*rand(1,dm(3));    
    
    P(it).IIR = check_stability(P(it).IIR);
    
end

end


%C: clones population to be mutated
%f: clones fitness vector
%C_hat: population after mutation
%dm: dimension vector
%rho: mutation parameter
function C_hat = hypermut(C,f,rho,dm)

Nc = length(C);
C_hat = allocate_pop(Nc);

p_mut = exp(-rho.*f); %mutation probability
for it=1:Nc
   C_hat(it).FIR = C(it).FIR + p_mut(it)*randn(1,dm(1)); 
   C_hat(it).IIR = C(it).IIR + p_mut(it)*randn(1,dm(2));
   C_hat(it).g = C(it).g + p_mut(it)*randn(1,dm(3)); 
   
   C_hat(it) = check_boundaries(C_hat(it));
   C_hat(it).IIR = check_stability(C_hat(it).IIR);
         
end

end

%replace the worst individuals
function Ab = replace(Ab,Abd,f)

Nabd = length(Abd);

[~,f_ind] = sort(f,'ascend');

Ab(f_ind(1:Nabd)) = Abd;

end


%fitness normalization
function [vnorm] = normfit(v)
nAB = length(v);
mn = min(v);
mx = max(v);

if(mn==mx)
    vnorm = 0.5.*ones(nAB,1);
else
    vnorm = (v-mn)./(mx-mn); %normalized fitness has to be maximized
end

end

%memory allocation
function M = allocate_pop(n)

M(n).FIR = []; M(n).IIR = []; M(n).g = [];


end

%validation routine for each member of the population
function p = check_boundaries(p_old)

global MAX_g MIN_g MAX_FIR MIN_FIR MAX_IIR MIN_IIR;

p = p_old;

upper_g = p.g>MAX_g;
lower_g = p.g<MIN_g;
p.g(upper_g) = MAX_g;
p.g(lower_g) = MIN_g;

upper_FIR = p.FIR>MAX_FIR;
lower_FIR = p.FIR<MIN_FIR;
p.FIR(upper_FIR) = MAX_FIR;
p.FIR(lower_FIR) = MIN_FIR; 

upper_IIR = p.IIR>MAX_IIR;
lower_IIR = p.IIR<MIN_IIR;
p.IIR(upper_IIR) = MAX_IIR;
p.IIR(lower_IIR) = MIN_IIR; 

end

%validation routine to check IIR stability (poles inside URC)
function iirnew = check_stability(iir)

wd = [1 iir]; %denominator

poles_wd = roots(wd);    
poles_out_uc = abs(poles_wd)>1;    
poles_wd(poles_out_uc) = poles_wd(poles_out_uc)./(abs(poles_wd(poles_out_uc))+.01); %poles inside URC
wd = poly(poles_wd);
wd = wd./wd(1);

iirnew = wd(2:end);

end



